using System;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace Photo_Editor_Application
{
    /// <summary>
    /// The EllipseFrameComponent class implements the
    /// functionality to draw an ellipse shaped frame.
    /// </summary>
    /// <requirement>F:image_graphics_special_effects</requirement>
    public class EllipseFrameComponent : GraphicsComponent
    {
        /// <summary>
        /// Constructor for circle component.
        /// </summary>
        public EllipseFrameComponent()
        {
        }

        /// <summary>
        /// Draws the circle in the provided Graphics object.
        /// Calculates the bouding box so width and height are 
        /// allways > 0.
        /// </summary>
        public override void Draw()
        {
            // Properties needed to draw the border.
            Region region;
            Rectangle outerRectangle;
            Rectangle innerRectangle = new Rectangle();
            // Create a graphics path object
            GraphicsPath gp = new GraphicsPath();
            int width =  CalcViewportCoords(EndPoint).X - CalcViewportCoords(StartPoint).X;
            int height = CalcViewportCoords(EndPoint).Y - CalcViewportCoords(StartPoint).Y;
            // Check if width and height are zero. 
            // If so set them to one.
            if (width == 0) width = 1;
            if (height == 0) height = 1;
            if(width < 0 && height < 0)
                outerRectangle = new Rectangle(CalcViewportCoords(EndPoint).X, 
                    CalcViewportCoords(EndPoint).Y, 
                    Math.Abs(width), 
                    Math.Abs(height));
            else if (width < 0)
                outerRectangle = new Rectangle(CalcViewportCoords(EndPoint).X, 
                    CalcViewportCoords(StartPoint).Y, 
                    Math.Abs(width), 
                    height);
            else if (height < 0)
                outerRectangle = new Rectangle(CalcViewportCoords(StartPoint).X, 
                    CalcViewportCoords(EndPoint).Y, 
                    width, 
                    Math.Abs(height));
            else 
                outerRectangle = new Rectangle(CalcViewportCoords(StartPoint).X, 
                    CalcViewportCoords(StartPoint).Y, 
                    width, 
                    height);

            // Calculate the border width and height.
            // If the Line Thickness is set to the maximum value 
            // of 20 then the whole rectangle is assumed to be border.
            int borderWidth = (int)(Math.Abs(width)*DrawingPen.Width)/40 ;
            int borderHeight = (int)(Math.Abs(height)*DrawingPen.Width)/40 ;
            // Set enclosing rectangle for inner ellipse
            // taking into account the border width.
            innerRectangle.X = outerRectangle.X + borderWidth;
            innerRectangle.Y = outerRectangle.Y + borderHeight;
            // Border is on both sides. Adjust width and height of
            // inner ellipse according.
            innerRectangle.Width = outerRectangle.Width - borderWidth*2;
            innerRectangle.Height = outerRectangle.Height - borderHeight*2;
		
            // Add the enclosing Rectangle to the 
            // GraphicsPath.
            gp.AddEllipse(outerRectangle);
            // Use a PathGradientBrush to fill the region.
            PathGradientBrush pgb = new PathGradientBrush(gp);
            // Set the center color to Transparent = Alpha equals 0.
            pgb.CenterColor = Color.Transparent;
            // More than one surround color can be defined.
            // In this example the coloris set to the
            // value the user selected or black as default. 
            pgb.SurroundColors = new Color[]
        {
            DrawingPen.Color
        };
            // Create a region the size of the ouline rectangle
            region = new Region(gp);
            // reset the graphics path.
            gp.Reset();
            // add the inner ellipse to the path
            gp.AddEllipse(innerRectangle);
            // subtract the inner ellipse from
            // the region by doing Xor.
            // The result is just the border with the specified size.
            region.Xor(gp);
            // Fill the region using the PathGradientBrush
            DeviceContext.FillRegion(pgb, region);
            // Dispose the brus, graphics path and the region
            region.Dispose();
            gp.Dispose();
            pgb.Dispose();
        }
    }

}
